@@ -1,3 +1,7 @@
+"""
+
+"""
+
 """
 websocket - WebSocket client library for Python
 
@@ -15,8 +19,7 @@ Copyright (C) 2010 Hiroki Ohtani(liris)
 
     You should have received a copy of the GNU Lesser General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1335  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
 """
 import errno
@@ -24,7 +27,6 @@ import select
 import socket
 
 import six
-import sys
 
 from ._exceptions import *
 from ._ssl_compat import *
@@ -62,7 +64,10 @@ def setdefaulttimeout(timeout):
     """
     Set the global timeout setting to connect.
 
-    timeout: default socket timeout time. This value is second.
+    Parameters
+    ----------
+    timeout: int or float
+        default socket timeout time (in seconds)
     """
     global _default_timeout
     _default_timeout = timeout
@@ -70,7 +75,12 @@ def setdefaulttimeout(timeout):
 
 def getdefaulttimeout():
     """
-    Return the global timeout setting(second) to connect.
+    Get default timeout
+
+    Returns
+    ----------
+    _default_timeout: int or float
+        Return the global timeout setting (in seconds) to connect.
     """
     return _default_timeout
 
